Subroutine GetInputFile (IUnit, filename)

    implicit none
    integer , intent (in) :: IUnit
    character(*) filename
    integer :: ierror, K, N

    ierror = 1
    K = 0
    DO WHILE (ierror /= 0 .AND. K < 3) 
        CALL AskQuestion(filename, " ", " ", N, "Input File? ")
        K = K + 1
        IF (N > 0) THEN
            OPEN (IUnit, FILE = filename, &
                    STATUS = "OLD", ACTION = "READ", IOSTAT=ierror)
            IF (ierror /= 0) THEN
            !allow that input file has default extension "txt", so
            !try again to see if the user misses extension "txt"
            !in the file name entered:
                OPEN (IUnit, FILE = filename(1:N) // ".txt", &
                        STATUS = "OLD", ACTION = "READ", IOSTAT=ierror)
                IF (ierror /= 0) then
                    WRITE (*, 90) filename(1:N), filename(1:N)//".txt"
                else
                    filename(N+1:N+4) = ".txt"
                end if
            END IF
        END IF
    END DO
    IF (ierror /= 0) THEN
        WRITE (*, 95) K
        STOP
    END IF
95  FORMAT (1X, "NO INPUT FILE IS GIVEN AFTER", I3, " TRIES! PROGRAM ENDS!")
90  FORMAT (1X, "FILE ", A, " OR ", A, " DOES NOT EXIST!")

End Subroutine GetInputFile

!------------------------------------------------------------------
Subroutine Version (IOut, filename)

    integer :: IOut
    character (*) :: filename
    character (12) :: Mydate, Mytime
    character (40) :: Info
    
    Info = "Version 2.0   Modified September 2013"
    call date_and_time(Mydate, Mytime) ! positional arguments
    WRITE (IOut, 100) Info(1:Len_Trim(Info))
    write(IOut, 110) Mydate(5:6),Mydate(7:8), Mydate(1:4), &
            Mytime(1:2), Mytime(3:4), Mytime(5:6)
    Write (IOut, 120) filename(1:Len_Trim(filename))
 
100 FORMAT (5X, "Output From Program AgeNe by C. Do and R. Waples", &
             /,12X, A)
110 FORMAT (12X, "Date: ", 2(A, "/"), A, ", Time: ", 2(A, ":"), A,/)
120 FORMAT (12X, "Input File: ", A, /)

End Subroutine Version

!------------------------------------------------------------------
Subroutine Dashes (IOut, m)

    integer, intent(in) :: IOut, m
    integer :: n
    do n=1, m
        if (n<m) then
            write (IOut, 100, advance="no") "-"
        else
            write (IOut, 100) "-"
        end if
    end do
100 format (A)

End Subroutine Dashes
!------------------------------------------------------------------


Subroutine GetInputType(LineSkipped, Line, IUnit, IOut, model, NPop, &
    maxage, initialrm, ierror, xfile)
! Read from input file IUnit.
! The first LineSkipped lines in IUnit, put those lines within two dashed
! lines in output file IOut.
! After LineSkipped lines will be the real input lines.
! if xfile (output parameter) is true, extra output file should be created.
! On output parameter Line is updated, by adding the number of lines read
! from IUnit
    implicit none
    integer, intent(in) :: LineSkipped, IUnit, IOut
    integer, intent(inout) :: ierror, Line
    integer, intent(out) :: maxage, NPop, model
    real, intent (out) :: initialrm
    logical, intent (out) :: xfile

    integer :: J, m, NextSkip
    character(200) filename, mystr
    character(20) info
    character reply
    logical :: Success = .true.

    initialrm = 1.0
    xfile = .false.

    ierror = 0
! skip LineSkipped lines, those lines will be considered as headlines for outputs,
! and will be put within two dashed lines
    do j = 1, LineSkipped
        Line = Line + 1
        read (IUnit, 100, IOSTAT=ierror) mystr
        if (ierror <> 0) return
        m = len_trim(mystr)
        if (j==1) then
            write (IOut, 100)
            call Dashes (IOut, m)
        end if
        write (IOut, 100) mystr(1:m)
    end do
    call Dashes (IOut, m)
! Read the first input line mystr:
!--------------------------------
    read (IUnit, 100, IOSTAT=ierror) mystr
    if (ierror <> 0) return
    NextSkip = 0
! The first block of mystr (info) should be maxage:
    call GetNextDat(mystr, info, Success, NextSkip, ierror)
!    print*,'info=', info
!    print*,'NextSkip=', NextSkip
    ierror = 0
    read (info, *, IOSTAT = ierror) maxage
    if (ierror <> 0) return
! The second block of mystr (info) should be NPop
    call GetNextDat(mystr, info, Success, NextSkip, ierror)
!    print*,'info=', info
!    print*,'NextSkip=', NextSkip
    ierror = 0
    read (info, *, IOSTAT = ierror) NPop
    if (ierror <> 0) return
! The third block of nystr (info) when capitalized, could be:
! * A letter 'X' or none at all, then it is Haploid model, and the 'X' if it
!   exists, is an instruction to put detailed output in a separate file.
!   (model = -1)
! * A word "FEMALE" or "MALE" signifying that this is a two-sex model
!   with sex change "female -> male" if the word is "FEMALE".
!   (model = 1)
! * A number r: then it is two-sex model without sex change, and
!   this r is the output parameter initialrm, initial fraction male.
!   (model = 0)
! For model = 0 or 1, look for the 4th block in mystr to see if a letter
! "X' is present. If it its, xfile is set to true.
    call GetNextDat(mystr, info, Success, NextSkip, ierror)
    m = len_trim(info)
    call Ucase(info)
    j = len_trim(info)

    WRITE (*, 100)
    if (m == 0 .or. info(1:j) == 'X') then
        model = -1
!        if (info(1:j) == "X") xfile = .true.
        write (*, 11) 'Haploid', maxage, NPop
    else ! two sexes, set model >= 0, = 0 when no sex change
        if (info(1:m) == "FEMALE") then
            model = 1
            write (*, 11) 'Female -> Male', maxage, NPop
            write (IOut, 100) 'Female -> Male Model'
        else if (info(1:m) == "MALE") then
            model = 2
            write (*, 11) 'Male -> Female', maxage, NPop
            write (IOut, 100) 'Male -> Female Model'
        else
            model = 0
        !read initial fraction male
            read (info, *, IOSTAT = ierror) initialrm
            if (ierror <> 0) go to 99
            write (*, 11) 'No Sex Change', maxage, NPop
        end if
        call GetNextDat(mystr, info, Success, NextSkip, ierror)
        j = len_trim(info)

    end if
    if (info(1:j) == "X") xfile = .true.
99  Line = Line + 1
11  format(1x,A, ' Model, Max Age = ', I3, ', N1 =', I6)
100 format (A)

End Subroutine GetInputType

!------------------------------------------------------------------
Subroutine AskQuestion(filename, logname, InpName, Leng, Sentence)
! Sentence: a question to ask the user on screen, will be input/utput file
! filename : name of the file the user will enter
! Leng: length of filename
! logname and InpName are strings that filename should avoid
    Character (*):: filename, Sentence, logname, InpName
    Integer :: Leng
    character (200) :: reply
    logical :: done
    
    PRINT*
    done = .false.
    do while (.not. done)
        WRITE (*, 20, advance = "no") Sentence
        READ (*, 10) filename
        filename = ADJUSTL(filename)
        Leng = LEN_TRIM(filename)
        Reply = filename
        call UCase (Reply)
        call UCase (logname)
        call UCase (InpName)
! Note: logname could be an empty string, so need Reply /= " "
        if (((Reply.eq. logname) .or. (Reply .eq. InpName)) .and. Reply /= " ") then
            write (*, 11) "Please choose another name"
        else
            done = .true.
        end if
    end do
!    WRITE (*, 10)

10  FORMAT (A)
11  FORMAT (1x,A)
20  FORMAT (" >> ", A)
30  FORMAT (/, 4X, A)

End Subroutine AskQuestion
!------------------------------------------------------------------

SUBROUTINE GetNextDat(Line, BlockDat, Success, NextSkip, ierror)
!from string Line, after the position NextSkip, get a block of "visible"
!characters to form BlockDat as the return value, set Success = true if such
!block is found. NextSkip will be assigned as the position after the last
!character of BlockDat.
    CHARACTER (*), INTENT (OUT) :: BlockDat
    INTEGER, INTENT (INOUT) :: ierror, NextSkip 
                    !NextSkip: # chars to skip for the next call GetNextDat,
                    !reading of input to get value of "Line"
    CHARACTER (*) :: Line
    LOGICAL, INTENT (OUT) :: Success
    
    INTEGER :: J, K, N
    CHARACTER :: CH
    LOGICAL :: Visible

    BlockDat = " "
    Success = .True.

    N = LEN_TRIM (Line)
    Visible = .FALSE.
!find the first position on string LINE, which is a visible char
!Blank char has ICHAR value = 32, TAB has value less (=9).
!All invisible ASCII chars have value < 32.
!DEL key has value 127, no ASCII char has value > 127.
!(there are 47 keys - letters, digits, signs - on keyboard, and by
!using Shift key, total number of those chars is 94 = 126 - 32.)
!    DO J = 1, N !this loop is skipped (after j is set 1) if N = 0
    DO J = NextSkip+1, N !this loop is skipped (after j is set 1) if N = 0
        CH = Line (J:J)
        Visible = (IACHAR(CH) > 32 .AND.  IACHAR(CH) < 127)
        IF (Visible) EXIT
    END DO
!out of this loop: (J <= N  <=>  Visible = true)
!       If "LINE" has only blanks, we assume the rest of input line
!       is empty, we're done with current input line:
    IF (.NOT. Visible) THEN
        ierror = 0
        Success = .false.
        RETURN
    END IF
    DO K = (J+1), N !if J = N, loop is skipped, then K = N+1
        CH = Line (K:K)
        Visible = (IACHAR(CH) > 32 .AND. IACHAR(CH) < 127)
        IF (.NOT. Visible) EXIT
    END DO
!out of this loop, (K <= N <=> visible=false.  K = N+1 <=> visible=true)
!either
!    CH is invisible: then K-J is the # of consecutive visible chars
! or
!    CH is visible: LINE is visible from J to the end:
!        then K-J = N+1-J is also the # of consecutive visible chars
    IF (K <= N) THEN
        NextSkip =  K
!        NextSkip = NextSkip + K
    ELSE
        NextSkip =  N
!        NextSkip = NextSkip + N
    END IF
    K = K - 1   !if visible=true, K = N+1, reassigned to be N
                !Now from J to K, Line includes only "visible" chars
    BlockDat = Line (J:K)

END SUBROUTINE GetNextDat
!------------------------------------------------------------------
Subroutine PrintSumTable (favorite, IOut, maxage, NPop, s, b, Lx, bscaled, p)

    real, intent(in) :: s(0:maxage), Lx(0:maxage), p(maxage), &
                    b(maxage), bscaled(maxage)
    character*(*) favorite
    real :: bxLx, bscaledLx, Bx, xBxNx, sumbxLx, sumbscaledLx, sumBx, sumxBxNx
    integer :: Nx, Nsum
    character (200):: fmtstr1, fmtstr2, fmtstr3

!    fmtstr1 = "(80('_'),/,/,'Age(x)', 1x, 'sx', 7x, 'bx', 4x, 'lx', 9x, 'bxlx'," &
!        // "5x, 'bx''', 4x, 'bx''lx', 6x, 'Nx', 6x, 'Bx', 4x, 'xBx/N1',/, 80('-'))"
!    fmtstr2 = "(I3,2x, f4.2,1x,f9.1,1x,f6.3,1x, f11.2, 1x, 2(f7.3,1x)," &
!        // " I7, 1x, f8.1, 1x, f8.3)"
!    fmtstr3 = "(80('_'),/,/, 13x, 'Totals', 5x, f14.2, 1x, f15.3, 1x, " &
!        // "I7,1x, I6, 1x, f10.3,/)"

    fmtstr1 = "(77('_'),/,/,'Age(x)', 1x, 'sx', 7x, 'bx', 4x, 'lx', 7x, 'bxlx'," &
        // "5x, 'bx''', 4x, 'bx''lx', 6x, 'Nx', 6x, 'Bx', 3x, 'xBx/N1',/, 77('-'))"
    fmtstr2 = "(I3,2x, f4.2,1x,f9.1,1x,f6.3,1x, f9.2, 1x, 2(f7.3,1x)," &
        // " I7, 1x, f8.1, 1x, f7.3)"
    fmtstr3 = "(77('_'),/,/, 13x, 'Totals', 3x, f14.2, 1x, f15.3, 1x, " &
        // "I7,1x, I6, 1x, f9.3,/)"

    write (IOut, 10) favorite(1:len_trim(favorite))
!    write (IOut, 100)
    write (IOut, fmtstr1)
    sumbxLx = 0
    sumbscaledLx = 0
    sumBx = 0
    sumxBxNx = 0
    Nsum = 0
    do j = 1, maxage
        bxLx = b(j)*Lx(j)
        bscaledLx = bscaled(j)*Lx(j)
        Nx = NInt(NPop*p(j))
        Bx = NPop*bscaled(j)*p(j)
        xBxNx = j*bscaled(j)*p(j)
        sumbxLx = sumbxLx + bxLx
        sumbscaledLx = sumbscaledLx + bscaledLx
        sumBx = sumBx + Bx
        sumxBxNx = sumxBxNx + xBxNx
        Nsum = Nsum + Nx
!        write (IOut, 200) j, s(j), NInt(b(j)), Lx(j), bxLx, bscaled(j), &
        write (IOut, fmtstr2) j, s(j), b(j), Lx(j), bxLx, bscaled(j), &
                            bscaledLx, Nx, Bx, xBxNx
    end do
!    write (IOut, 300) sumbxLx, sumbscaledLx, Nsum, NINT(sumBx), sumxBxNx
    write (IOut, fmtstr3) sumbxLx, sumbscaledLx, Nsum, NINT(sumBx), sumxBxNx

10  format(/,A)
!100 format (80("_"),/,/,"Age(x)", 2x, "sx", 7x, "bx", 3x, "lx", 10x, "bxlx", &
!        4x, "bx'", 4x, "bx'lx", 6x, "Nx", 6x, "Bx", 4x, "xBx/N1",/, 80("-"))
!200 format (I3,3x, f4.2,1x,I8,1x,f6.3,1x, f11.3, 1x, 2(f7.3,1x), I7, 1x, f8.1, 1x, f8.3)
!300 format (80("_"),/,/, 13x, "Totals", 5x, f14.3, 1x, f15.3, 1x, I7,1x, I6, &
!            1x, f10.3,/)

End Subroutine PrintSumTable

!------------------------------------------------------------------
Subroutine PrintParam (IOut, model, NPop, maxage, initialrm, poissOne, poissTwo)

    real, intent(in) :: initialrm, poissOne(maxage), poissTwo(maxage)
    integer, intent(in) :: IOut, NPop, model, maxage
    character *(24) :: mystr, fmtstr

    write (IOut, 20)

    mystr = "N1 ="
        write (IOut, 200) mystr, NPop
    if (model == 0) then
        mystr = "Initial sex ratio ="
        write (IOut, 100) mystr, initialrm
    end if
! this poissOne is just one value of the poisson factors. May add more later
! to include all values of poissOne and poissTwo
    if (model >= 0) then
        mystr = "Poisson factor (female):"
    else
        mystr = "Poisson factor:"
    end if
    fmtstr = "(A,    (f9.1,1x))"
    !         12345678
    write (fmtstr(4:7),"(I3)") maxage
        write (IOut, fmtstr) mystr, (poissOne(j), j=1,maxage)
    mystr = "Poisson factor (male):"
        if (model >= 0) write (IOut, fmtstr) mystr, (poissTwo(j), j=1,maxage)

20  format (/,"Input Parameters:",/,17("-"),/)
100 format (A, f11.3)
200 format (A, I7)

End Subroutine PrintParam

!------------------------------------------------------------------
! add in June 2013, to help in format printing
SUBROUTINE GETLEN (mystr, minLen)
! on output, minLen will change to the length of mystr without leading or
! trailing invisible characters, if that length is larger than minLen;
! otherwise, no change in minLen
    CHARACTER (*), INTENT(INOUT) :: mystr
    INTEGER, INTENT(INOUT) :: minLen
    CHARACTER :: c
    INTEGER :: J, K, M, N
    LOGICAL :: visible
    N = LEN(TRIM(mystr))
    IF (N .LE. minLen) RETURN
    visible = .FALSE.
    K = 0
    DO WHILE (.NOT. visible)
        K = K + 1
        c = mystr (K:K)
        visible = (IACHAR(c) > 32 .AND. IACHAR(c) < 127)
    END DO
    N = N - K + 1   !N now is the length of mystr without leading invisible chars
    if (minLen < N) minLen = N
    RETURN
END SUBROUTINE GETLEN

!------------------------------------------------------------------

Subroutine PrintSummary (IOut, maxage, NPop, Vm, Vf, Vk, Ne, NeLong, rNeN, &
                        rNeA, TotalN, TotalF, TotalM, Adult, AdultF, AdultM, &
                        km, kf, kbar, & !next line of parameters is for annual:
                        Nb, Nbf, Nbm, kbarM, kbarF, kBarYr, VkM, VkF, VkYr, &
                        Generation, model, indStop, poissOne, poissTwo)

    real, intent(in) :: Vk, Ne, NeLong, rNeN, rNeA, TotalN, Adult, AdultF, &
                    AdultM, Generation, km, kf, kbar, &
                    Nb, Nbf, Nbm, kbarM, kbarF, kBarYr, VkM, VkF, VkYr, &
                    poissOne(maxage), poissTwo(maxage)
    integer, intent(in) :: IOut, NPop, model, maxage, indStop
    character *(24) :: mystr, fmtstr, num1, num2
    logical :: SamePoiss
    integer :: i, j
    integer :: m, n

    num1 = "";
    num2 = "";
    write (IOut, 30)
! modify in June 2013 for adding "(Female, Male)":
    if (model == 0) then
        ! Find m to be the smallest length that fit to both TotalF, AdultF:
        m = 1
        write (num1, "(I8)") NINT(TotalF)
        write (num2, "(I8)") NINT(AdultF)
        CALL GETLEN(num1, m)
        CALL GETLEN(num2, m)
        ! Find n to be the smallest length that fit to both TotalM, AdultM:
        n = 1
        write (num1, "(I8)") NINT(TotalM)
        write (num2, "(I8)") NINT(AdultM)
        CALL GETLEN(num1, n)
        CALL GETLEN(num2, n)
        WRITE (fmtstr, "(2(4X,I2))") m, n
        fmtstr(1:4) = "(A,I"
        fmtstr(7:10)= ",A,I"
        fmtstr(13:15) = ",A)"
        num1 = ""
        num2 = ""
        WRITE (num1, fmtstr) "     (", NINT(TotalF), ", ", NINT(TotalM), ")"
        WRITE (num2, fmtstr) "     (", NINT(AdultF), ", ", NINT(AdultM), ")"
        mystr = "Total N (Female, Male) ="
        write (IOut, 200) mystr, NINT(TotalN), num1
        mystr = "Adult N (Female, Male) ="
        write (IOut, 200) mystr, NINT(Adult), num2
    else
        mystr = "Total N ="
        write (IOut, 200) mystr, NINT(TotalN)
        mystr = "Adult N ="
        write (IOut, 200) mystr, NINT(Adult)
    end if
    mystr = "Generation length ="
        write (IOut, 100) mystr, Generation
!    mystr = "N1 ="
!        write (IOut, 200) mystr, NPop
    write (IOut, 5)
    write (IOut, 5) "               LIFETIME"
    write (IOut, 5) "               --------"
    if (model == 0) then
        mystr = "kbar(female)  ="
        write (IOut, 100) mystr, kf
        mystr = "kbar(male)    ="
        write (IOut, 100) mystr, km
        mystr = "kbar(overall) ="
        write (IOut, 100) mystr, kbar
        mystr = "Vk(female)    ="
        write (IOut, 100) mystr, Vf
        mystr = "Vk(male)      ="
        write (IOut, 100) mystr, Vm
        mystr = "Vk(overall)   ="
        write (IOut, 100) mystr, Vk
        
        write (IOut, 5)
        mystr = "Ne (Eq 2)  ="
        write (IOut, 300) mystr, Ne
        mystr = "Ne (Eq A1) ="
!check it Poisson factors are the same:
        SamePoiss = .true.
chkPoiss: &
        do i=1, maxage-1
            if (PoissOne(i) /= PoissTwo(i)) then
                SamePoiss = .false.
                exit chkPoiss
            end if
            do j=i+1, maxage
                if (PoissOne(i) /= PoissOne(j) .or. PoissTwo(i) /= PoissTwo(j)) then
                    SamePoiss = .false.
                    exit chkPoiss
                end if
            end do
        end do chkPoiss
        
        if (SamePoiss) then
            write (IOut, 300) mystr, NeLong
        else
            write (IOut, 5, advance = "no") mystr
            write (IOut, 5) "Not calculated when Poisson factor varies with age"
        end if
        mystr = "Ne (Eq 2)/(Total N) ="
        write (IOut, 100) mystr, rNeN
        mystr = "Ne (Eq 2)/(Adult N) ="
        write (IOut, 100) mystr, rNeA
    else
        mystr = "Vk ="
        write (IOut, 100) mystr, Vk
        mystr = "Ne ="
        write (IOut, 300) mystr, Ne
        mystr = "Ne/(Total N) ="
        write (IOut, 100) mystr, rNeN
        mystr = "Ne/(Adult N) ="
        write (IOut, 100) mystr, rNeA
    end if
    write (*, 50) "Ne =", NINT(Ne)
    write (IOut, 5)
    write (IOut, 5) "               ANNUAL"
    write (IOut, 5) "               ------"
    mystr = "Nb ="
    if (model >= 0) then
        write (IOut, 5) "FEMALE:"
        mystr = "      kbar ="
        write (IOut, 100) mystr, kbarF
        mystr = "      Vk   ="
        write (IOut, 100) mystr, VkF
        mystr = "      Nb   ="
        write (IOut, 300) mystr, Nbf
!        write (IOut, 200) mystr, NINT(Nbf)
        write (IOut, 5) "MALE:"
        mystr = "      kbar ="
        write (IOut, 100) mystr, kbarM
        mystr = "      Vk   ="
        write (IOut, 100) mystr, VkM
        mystr = "      Nb   ="
        write (IOut, 300) mystr, Nbm
!        write (IOut, 200) mystr, NINT(Nbm)
        write (IOut, 5) "Overall:"
        mystr = "      kbar ="
        write (IOut, 100) mystr, kBarYr
        mystr = "      Vk   ="
        write (IOut, 100) mystr, VkYr
        mystr = "      Nb   ="
        write (IOut, 300) mystr, Nb
!        write (IOut, 200) mystr, NINT(Nb)
        write (*, 300) " Nb per year =", Nb
!        write (*, 300) " (by taking harmonic mean of 2Nbm, 2Nbf) =", &
!                        (4 * Nbf * Nbm)/(Nbf + Nbm)
    end if
    if (indStop > 0 .and. indStop < maxage) write (IOut, 90) indStop+1, indStop
    write (IOut, 5)


5  format (A)
30  format (//,"Outputs:",/,8("-"),/)
50  format (1x, A, I7)
100 format (A, f11.3)
!200 format (A, I7, 5A)
200 format (A, I7, A)
300 format (A, f9.1)
90  format (//,"Predetermined fraction male at age ", I3, &
            " does not allow sex change.", /,"Sex change stops at age", I3)

End Subroutine PrintSummary

!------------------------------------------------------------------


SUBROUTINE OpenNewFile (IUnit, Prefix, FileName, Purpose)

    INTEGER :: IUnit
    CHARACTER (*), INTENT (IN) :: Prefix, FileName, Purpose
    
    INTEGER :: ierror = 0
    
    WRITE (*, 10) Purpose, " are written to file ", TRIM(FileName)
!sometimes, if including ACTION = "WRITE", will give ierror /= 0
!when this is called the second time for shorter output file
!    OPEN (IUnit, FILE = FileName, STATUS = "NEW", &
!            ACTION = "WRITE", IOSTAT=ierror)
    OPEN (IUnit, FILE = Prefix // FileName, STATUS = "NEW", IOSTAT=ierror)
    IF (ierror /= 0) THEN
        WRITE (*, 20) "(existed, will be overwritten)"
!        OPEN (IUnit, FILE = FileName, STATUS = "UNKNOWN", &
!                    ACTION = "WRITE", IOSTAT=ierror)
        OPEN (IUnit, FILE = Prefix // FileName, STATUS = "UNKNOWN", IOSTAT=ierror)
    ENDIF
10  FORMAT (4X, 3A)
20  FORMAT (29X,A)

END SUBROUTINE OpenNewFile

