!-------------------------------------------------------------------------

Subroutine LeslieOne(s1, b1, L1, G1, Bscaled, maxage, &
                    TotalN, Adult, totGeneration, p)
! This sub is for one-sex case only
    integer, intent(in) :: maxage
    real, intent(in) :: s1(0:maxage), b1(maxage)
! s should have s(0) = 1 at input
    real, intent(out) :: L1(0:maxage), G1(maxage), Bscaled(maxage), &
                TotalN, Adult, totGeneration, p(maxage)
    real :: bt, sumb, tsumb
    integer :: t

    p = 1
    L1 = 1
    G1 = 1
    TotalN = 0
    Adult = 0
    sumb = 0
    tsumb = 0
    do t = 1, maxage
        L1(t) = L1(t-1)*s1(t-1)
        if (t > 1) p(t) = p(t-1)*s1(t-1)
        G1(t) = L1(t)*(1 - s1(t))
        TotalN = TotalN + p(t)
        if (b1(t)>0) Adult = Adult + p(t)
        bt = b1(t)*p(t)
        sumb = sumb + bt
        tsumb = tsumb + t*bt
    end do
    totGeneration = tsumb/sumb
!    sumgt = 0
    do t = 1, maxage
!        Bscaled(t)= b1(t)/(2*sumb)
!The following matches with Leslie5, and by setting parameter growth = 2:
!        Bscaled(t)= 2* b1(t)/sumb
        Bscaled(t)= b1(t)/sumb
!sumgt = sumgt+G1(t)
!print*,'G(t)=',G1(t)
    end do
!print*,'sum G=',sumgt

End Subroutine LeslieOne

!----------------------------------------------------------------------------

Subroutine LeslieCalcul(pM, pF, s, b, Bmix, rm, maxage, L, totGeneration, & !9
             indStop, Q, G, poisson, PoissonMix, TotalN, TotalF, TotalM, &  !8
             Adult, AdultF, AdultM, rmTot, rmAdult, Bscaled, model, &   !7
             growth, kBarYr, kbarF, kbarM, VkYr, VkF, VkM, ILog)    !8
! Note that, as in other subroutines, all population size TotalN, Adult, etc,
! are scaled based on the initial population size = 1.
    implicit none
    integer, intent(in) :: maxage, model, ILog

    real, intent(in) :: s(2,0:maxage), b(2,maxage), poisson(2,maxage), growth
    real :: rm(maxage)
                        !rm: input if model = 0.
    real,intent(out) :: Bmix(0:maxage,maxage), PoissonMix(0:maxage,maxage), &
                Adult, AdultF, AdultM, TotalN, TotalF, TotalM, rmTot, rmAdult, &
                Bscaled(2,maxage), Q(0:maxage), L(2,0:maxage), pM(maxage), &
                kBarYr, kbarF, kbarM, VkYr, VkF, VkM, & ! <- these are for "Annual"
                pF(maxage), G(0:maxage,maxage), totGeneration
! TotalN, Adult, rmTot, rmAdult: total pop, total adults,
! male fractions of TotalN, Adult
    integer, intent(out):: indStop       !the age where probability of sex
! change is still > 0 (then <= 0 afterwards); use it as a flag to determine
! if changing sex is to be stopped, or deliver an error message.


! The following are arrays of two dimensions, the first is for sex:
! index 1 is for female, 2 for male.
! s : survival rates for females and males (should have s(:,0)=1 at input)
! L : accumulated survival rates.
! b : offsprings born by females, males

! rm : Fraction males

! If model = 0, no change sex, the original fraction male is given as rm(1),
! subsequent fraction males will be determined by survivor rates 

! Outputs: Will divide the population in groups
! Q(j) : fraction of group j relative to whole population
! G(j,t): fraction of individuals (in group j) that die at the end of age t.
! Bmix(j,t) : b scaled to be reproductive success for individual 
!           in group j at age t.
! Bmix(j,t) : Bmix * (appropriate poisson multiplier).

    integer:: t, j, k, m, n, ind1, ind2
    real :: c(0:maxage), p(0:maxage), smix(0:maxage)
!    real :: kbarM, kbarF 
    real :: rateM, rateF, rnext, Bf, Bm, sumBf, sumBm, tsumBf, &
            tsumBm, yf, ym, y2f, y2m, sumQ
!Add June 2013, for correction of Adult males, females calculations:
    integer :: gr, fGrown, mGrown   ! Age of female, male where begins to reproduce

! This sub is for two-sex only:
    if (model < 0) return
! model = 1 if sex change allowed from F to M, model = 2 for M to F.
! S(1,.) is for female, s(2,.) for male; similarly for L.
! Fraction male is input at each age t.
! model = 0: no sex change, only fraction male r(1) is needed,
! fraction male is determined in that case for t > 1.
!    Offspftot = 0
!    Offspmtot = 0
    
    tsumBf = 0
    tsumBm = 0
    sumBf = 0
    sumBm = 0
! change in June 2013, calculate for each sex, so growth is set to half:
    gr = growth/2
! L is accumulated survival rates of females and males    
    L = 1
! p(t): population at age t
    p = 1
! c(t): prob. of changing sex at age t
    c = 0
    smix = 1
!    Ngr = 0 !number of groups G. G(0) is group of opposite of favorite sex
            !only (e.g. if sex change is from F to M, then favorite sex is F,
            !and G(0) has only M), the last group is of favorite sex only.
            !In between, G(j) is for groups having favorite sex up to age j,
            !then opposite sex afterward.
    Q = 0   !proportion of groups G relative to the cohort
    ! indStop is where the maximum of the index j above,
    ! the maximum age that sex change happened.
    indStop = maxage
    ! c is the probability of changing sex
    c = 0
    c(maxage) = 1
    ! when sex change allowed for one sex to the other, the one that
    ! can change sex is called favorite sex.
    ! set c(0) to be the fraction for opposite of favorite sex:
    if (model == 1) then
        c(0) = rm(1)    ! rate of male at age 1
    else if (model == 2) then
        c(0) = 1 - rm(1)
    else
    ! if no sex change, model = 0, we assume that favorite sex is female,
        c(0) = rm(1)    ! so set rate of male at age 1
        indStop = 0
    end if
    TotalF = 0
    TotalM = 0
    AdultF = 0
    AdultM = 0

!    write(2, '(/a)') 'Prob not change sex:'

    ! Q(j) is the proportion of individuals that are of favorite sex up to
    ! age j, then opposite sex afterward. Assuming all individuals are
    ! of opposite sex of the favorite one, then Q(0) should be proportion
    ! of opposite sex beginning at age 1, Q(maxage) is the proportion of
    ! individuals being of favorite sex and never change sex.
    Q(0) = c(0)
    sumQ = Q(0)
    
    if (Ilog>0) then
        if (model > 0) then
            if (model == 1) then
                write(ILog, 10) "Female -> Male"
            else
                write(ILog, 10) "Male -> Female"
            end if
            write(ILog, 10) "          Fraction Males"
            write(ILog, 10) "Year   Current    Designated   Sex-Change"
        else
            write(ILog, 10) "Year   Fraction Males"
        end if
    end if
    fGrown = maxage+1
    mGrown = maxage+1
    Do t = 1, maxage
        L(1,t) = L(1,t-1)*s(1,t-1)
        L(2,t) = L(2,t-1)*s(2,t-1)
        p(t) = p(t-1)*smix(t-1)
        if (t > 1) then
            !only consider changing sex if it was not stopped at previous t
            if (t <= indStop) then !this condition will not hold for model=0,
            !since indStop was set 0; so only apply to sex-change
                if (model == 1) then       !sex change OK from female to male
                    c(t-1) = rm(t) - (1-rm(t))*rnext
                else if (model == 2) then  !sex change OK from male to female
                    c(t-1) = 1 - rm(t) - rm(t)/rnext
                end if
                if (Ilog > 0) write(ILog, 20, advance = "no") &
                                    t, rateM/smix(t-1), rm(t)
                if (c(t-1) <= 0) then ! stop sex change from age t and beyond
! Use strict inequality:
!                if (c(t-1) < 0) then ! stop sex change from age t and beyond
                    c(t-1) = 0
                    indStop = t - 1
                    rm(t) = rateM/smix(t-1)
                    if (Ilog >0 .and. model >0) write(ILog, 10) "        NO"
                else
!                    Ngr = Ngr + 1   !number of groups increases
                    if (Ilog > 0 .and. model > 0) then
                        if (c(t-1)>0) then
                            write(ILog, 10) "       YES"
                        else
                            write(ILog, 10) "        X"
                        end if
                    end if
                end if

            else    ! no more sex change, fraction male is recalculated
                    ! if model > 0,
                    !(for model = 0, fraction male is known only at age 1)
                rm(t) = rateM/smix(t-1)
!    if (Ilog>0) write(ILog,'(a,I3,a, f12.4)') &
!                   "t>indStop: t=", t,", fraction=", rm(t)
!                if (Ilog>0) write(ILog,'(a,I3,a, f12.4)') &
!                   "No sex change, Year t =", t, ", fraction male =", rm(t)
                if (Ilog > 0) then
                    if (model > 0) then
                        write(ILog, 30) t, rm(t), "IGNORED", "NO"
                    else
                        write(ILog, 30) t, rm(t)
                    end if
                end if
!                if (Ilog>0) write(ILog,'(a,I3,a, f12.4)') &
!                   "No sex change, Year t =", t,  ", fraction male =", rm(t)

            end if

            Q(t-1) = (1.0 - sumQ)*c(t-1)  !probability of an idividual that
            !keeps its sex until age (t-1) then change sex at age t.
            !This will be 0 if sex change stops at this t or stopped before.
            sumQ = sumQ + Q(t-1)
        end if
        pF(t) = p(t)*(1-rm(t))
        pM(t) = p(t)*rm(t)
        TotalF = TotalF + pF(t)
        TotalM = TotalM + pM(t)
! Modify June 2013, Adult count increases according to the age where begin
! to give birth (fGrown, mGrown):
!        if (b(1,t)>0) AdultF = AdultF + pF(t)
!        if (b(2,t)>0) AdultM = AdultM + pM(t)
        if (b(1,t)>0 .AND. fGrown > t) fGrown = t
        if (b(2,t)>0 .AND. mGrown > t) mGrown = t
        if (t >= fGrown) AdultF = AdultF + pF(t)
        if (t >= mGrown) AdultM = AdultM + pM(t)
        Bf = b(1,t)*pF(t)    !total born by females
        Bm = b(2,t)*pM(t)    !total born by males
        sumBf = sumBf + Bf
        sumBm = sumBm + Bm
        tsumBf = tsumBf + t*Bf
        tsumBm = tsumBm + t*Bm
        ! the following are to be used by the next t:
        rateM = s(2,t)*rm(t)
        rateF = s(1,t)*(1-rm(t))
        smix(t) = rateM + rateF    !combined survival rate
        rnext = rateM/rateF     !predicted next fraction male
        
!        write(2,'(f10.6)') (1-c(t))
!    write(2,'(f10.4)') rm(t)
    End Do
    totGeneration = (tsumBf/sumBf + tsumBm/sumBm)/2.0
    Adult = AdultF + AdultM
    TotalN = TotalF + TotalM
    rmAdult = AdultM/Adult
    rmTot = TotalM/TotalN

!kbarM,kbarF,etc. are for calculating Effective number of Breeders Nb,
!kbarM for male, kbarF for female: these can be taken out later, unless for using
!in auxiliary outputs.
!Only need kBarYr, VkYr (kbar overall, Vk overall)
    
    kbarM = gr/TotalM
    kbarF = gr/TotalF
    kBarYr = 2*gr/TotalN
!print*
!print*, 'sumBm=', sumBm
!print*, 'sumBf=', sumBf
!print*, 'female mature age = ', fGrown
!print*, 'male mature age = ', mGrown
!print*, 'Total =', TotalN,  ', Male=', TotalM, ', Female=', TotalF
!print*, 'kb-male=', kbarM, ', kb-female=', kbarF, ', kb-overall=', kBarYr
!print*
!print*,'Bscaled of female, male:'

    VkYr = 0
! add June 2013:
    VkF = 0
    VkM = 0
    do t = 1, maxage
    ! count Bscaled for gametes, so don't divide by 2
        Bscaled(1,t)= b(1,t)/sumBf
        Bscaled(2,t)= b(2,t)/sumBm
!        Bscaled(1,t)= b(1,t)/(2*sumBf)
!        Bscaled(2,t)= b(2,t)/(2*sumBm)

!        yf = 2*Bscaled(1,t)-kBarYr
!        ym = 2*Bscaled(2,t)-kBarYr
!        y2f = 2*Bscaled(1,t)-kbarF
!        y2m = 2*Bscaled(2,t)-kbarM
        yf = Bscaled(1,t)-kBarYr
        ym = Bscaled(2,t)-kBarYr
        y2f = Bscaled(1,t)-kbarF
        y2m = Bscaled(2,t)-kbarM
!print*, 'ym = ', ym, ', y2m = ', y2m
!print*, 'yf = ', yf, ', y2f = ', y2f
! add june 2013:
        VkF = VkF + pF(t)*(Bscaled(1,t)*poisson(1,t) + y2f*y2f)
        VkM = VkM + pM(t)*(Bscaled(2,t)*poisson(2,t) + y2m*y2m)
        VkYr = VkYr + pF(t)*(Bscaled(1,t)*poisson(1,t) + yf*yf) + &
                pM(t)*(Bscaled(2,t)*poisson(2,t) + ym*ym)
!        VkF = VkF + pF(t)*(2*Bscaled(1,t)*poisson(1,t) + y2f*y2f)
!        VkM = VkM + pM(t)*(2*Bscaled(2,t)*poisson(2,t) + y2m*y2m)
!        VkYr = VkYr + pF(t)*(2*Bscaled(1,t)*poisson(1,t) + yf*yf) + &
!                pM(t)*(2*Bscaled(2,t)*poisson(2,t) + ym*ym)
    end do

    VkYr = VkYr/TotalN
    VkF = VkF/TotalF
    VkM = VkM/TotalM
!print*
!print*, 'Vk-overall=', VkYr
!print*, 'Vk-f =', VkF
!print*, 'Vk-m =', VkM
!print*

    !previous loop only gives values of Q(j) for j < maxage
    Q(maxage) = 1 - sumQ
! Now calculate the distribution of population in each age t
! in each group. If indStop < maxage, there is no group G(j)
! for j between indStop and maxage, for such j, Q(j) as calculated
! above should be 0
    G = 1
    if (model <= 1) then
    !favorite sex is female
        ind1 = 1
        ind2 = 2
    else
    !favorite sex is male
        ind1 = 2
        ind2 = 1
    end if
! Calculate G(j,t), the probability of an individual in group j
! that dies at the end of age t. Note that an individual in group j
! is of favorite sex up to age j, then change sex afterward.
! Favorite sex is indicated by ind1.
! Bmix represent progeny born by an individual at age t.
! That individual is of favorite sex at age t <= j, opposite sex if t > j.
    Do j = 0, maxage
        do t = 1, j
            G(j,t) = L(ind1, t)*(1 - s(ind1, t))
            Bmix(j, t) = Bscaled(ind1, t)
            PoissonMix(j, t) = poisson(ind1, t)
        end do
        do t = j+1, maxage
            G(j,t) = L(ind1, j+1)
            do m = j+1, t-1
                G(j,t) = G(j,t)*s(ind2,m)
            end do
            G(j,t) = G(j,t)*(1-s(ind2,t))
            Bmix(j, t) = Bscaled(ind2, t)
            PoissonMix(j, t) = poisson(ind2, t)
        end do
    End Do 
10  Format (A)    
20  Format (I3, 2(f12.6))
30  Format (I3, f12.6, 5x, A, 8x, A)

End Subroutine LeslieCalcul

!----------------------------------------------------------------------------

Subroutine SSDperGrp (maxage, frac, bmix, G, SSD, PoissonMix, model, &
                        growth, kbarPoisson, kbarAve, ILog, NPop)

    integer, intent(in) :: maxage, model, ILog
    real, intent(in) :: growth, frac, bmix(maxage), &
                        G(maxage), PoissonMix(maxage)
! growth: represents growth factor,
!        2 is for constant population in 2-sex case:
! the number of gametes produced per each sex
! bmix represents gametes
    real, intent(out) :: kbarPoisson, SSD, kbarAve

    real :: SSD1sum, y, SSy, SSD1(maxage), kbarN
    integer :: t

    real :: PopUsed(maxage), vt, kbar(maxage), SSD0(maxage), gametes
    
    if (frac <= 0) return
    kbarAve = 0
    sumb = 0

!use totpop to check if the sum of G(t) is 1, Poptot should be frac
!    Poptot = 0
    kbarPoisson = 0
    Do t = 1, maxage
!        gametes = 2*bmix(t)
! since bmix already represents gametes
! (at LeslieCalcul - called before this sub),
! or represents one progeny (in LeslieOne)
! so don't multiply by 2
        gametes = bmix(t)
        if (t == 1) then
            kbar(t) = gametes
            vt = gametes*PoissonMix(t)
        else
            kbar(t) = kbar(t-1) + gametes
            vt = vt + gametes*PoissonMix(t)
        end if
        PopUsed(t) = G(t) * frac
        kbarN = kbar(t) * PopUsed(t)
        SSD0(t) = vt * PopUsed(t)
        kbarPoisson = kbarPoisson + SSD0(t)
                    !expected of (Poisson factor)*(accumulated kbar)
        kbarAve = kbarAve + kbarN
!        Poptot = Poptot + PopUsed(t)
!print*,'Die at age',t,': ', PopUsed(t)
!print*,'k = ', kbar(t)
!print*,'vk = ', vt
    End Do
    kbarAve = kbarAve/frac
    kbarPoisson = kbarPoisson/frac
    SSD1sum = 0
    
!test:
!    SSysum=0
    
    Do t = 1, maxage
        y = kbar(t)-kbarAve
        SSy = y*y*PopUsed(t)
!test:        
!        SSysum = SSysum+SSy
        
        SSD1(t) = SSD0(t) + SSy
!test:        
!print*, 'SSD(1)=', SSD1(t)
        SSD1sum = SSD1sum + SSD1(t)
    End Do
!print*, 'sum of SSD(1)=', SSD1sum
    
    SSD = SSD1sum
    
!test:
!    print*,'fraction=', frac
!    print*,'SSySum=', SSysum

!this is revised, the increment of SSD applies to changing sex only:
    if (model > 0) then
        SSD = SSD + frac*((growth - kbarAve)*(growth - kbarAve))
    else if (model == 0) then
        SSD = SSD / frac
    end if
!
! if need to print output for the group, cover this return
!    return
    if (ILog == 0) return
!test:
!print*,'SSD=', SSD
    
    write (ILog, '(/,a,2x,f10.6,/,a,/)') 'Group fraction =', frac, &
    'List of values for: kbar, n, SSD0, SSD1'
!!    write (2, '(a,f18.6)')  'tot pop = ', Poptot
!    write (ILog, '(/a,f10.6)')  'kbarAve = ', kbarAve
    do j = 1, maxage
        write (ILog, '(4f12.6)')  kbar(j), PopUsed(j)*NPop, SSD0(j), SSD1(j)
    end do
    write (ILog, '(/2(a,f10.6))')  'kbarAve = ', kbarAve, &
                        ',   Sum of SSD1 = ', SSD1sum
!    write (ILog, '(/,6x,a,f28.6)')  'Sum of SSD1 = ', SSD1sum
    write (ILog, '(/,a,f28.6)')  'SSD for the group = ', SSD
    write (ILog, '(50("-"))')

End Subroutine SSDperGrp
!----------------------------------------------------------------------------

Real Function ScaledNe (totGeneration, Vk, model)

    integer, intent(in):: model
    real, intent(in):: totGeneration, Vk
    if (model > 0) then
        ScaledNe = (4*totGeneration)/(2+Vk)
    else
        ScaledNe = totGeneration/Vk
    end if
End Function ScaledNe

!----------------------------------------------------------------------------

Real Function VarXtoY (Vk, kbarPoisson, kx, kxy)

!factor represents a change of kbar, which is kxy/kx,
!here, x and y could represent male or female.
!For example, if x=m, y=f, then factor = kmf/km.
!If x=m, y=m, factor = kmm/km
!kbarPoisson is the expected value of progeny per one parent (male or female)
!with Poiison factor weighted in.

! VarXtoY calculates Var(m,m), Var(m,f), Var(f,f), Var(f,m), variances
! of male, female progeny by a male, and female, male progeny by a female.
    real, intent(in):: kbarPoisson, Vk, kx, kxy
    real :: factor
    factor = kxy/kx
    VarXtoY = factor*(kbarPoisson + factor*(Vk - kbarPoisson))
!print*,'Vk, kbarPoisson, kbar, kxy = ', Vk, kbarPoisson, kx, kxy
!print*,'Var = ', VarXtoY

End Function VarXtoY

!----------------------------------------------------------------------------

Real Function CoVarXY (Vk, kbarX, rx)


!CoVarXY calculates Cov(mm,mf), Cov(fm,ff),
!covariances of male, female progeny by a male, and by a female.
    real, intent(in):: Vk, kbarX, rx
! rx: initial fraction maleor female, assumed constant
    real :: factor
    factor = rx*(1-rx)
    CoVarXY = factor*(Vk - kbarX)
!print*,'Vk, kbar = ', Vk, kbarX
!print*,'Cov = ', covarxy

End Function CoVarXY

!----------------------------------------------------------------------------

Real Function CrossVar (Vm, Vf, kbarPoism, kbarPoisf, km, kf, kmm, &
                        kmf, kff, kfm, rm)

    real, intent(in):: Vm, Vf, kbarPoism, kbarPoisf, km, kf, kmm, &
                        kmf, kff, kfm, rm
    real :: A, B, factormf, factorfm
! A and B are the two terms enclosed in square brackets of formula (17), 
! p.287, Hill's Effective Size of Populations with Overlapping Generations,
! Theoretical Population Biology 3, 278-289 (1972). The formula is
!1/Ne = [1/(16*rm*L)]*A + [1/(16*(1-rm)*L)]*B

    factormf = rm/(1-rm)
!print*
!print*, 'Vmm, Cov(mm,mf), Vmf:'
    A = 2.0 + VarXtoY(Vm, kbarPoism, km, kmm) &
            + 2.0*factormf*CoVarXY(Vm, km, rm) &
            + factormf*factormf*VarXtoY(Vm, kbarPoism, km, kmf)
    factorfm = 1/factormf
!print*
!print*, 'Vff, Cov(fm,ff), Vfm:'
    B = 2.0 + VarXtoY(Vf, kbarPoisf, kf, kff) &
            + 2.0*factorfm*CoVarXY(Vf, kf, rm) &
            + factorfm*factorfm*VarXtoY(Vf, kbarPoisf, kf, kfm)
    CrossVar = ((1-rm)*A + rm*B)/(16*rm*(1-rm))
! Setting CrossVar this way so that CrossVar can play the role of Vk
! in the formula (scaled)Ne = L/Vk, where L is total generation, that is,
! Ne = L/(CrossVar).


End Function CrossVar

!----------------------------------------------------------------------------

Real Function Breeder(NPop, TotalN, factor, kbar, Vk, adjust)

    real, intent(in) :: TotalN, factor, kbar, Vk, adjust
    integer, intent(in) :: NPop

    Breeder = ((NPop*TotalN*factor*kbar) + adjust)/(kbar - 1 + (Vk/kbar))

End Function Breeder

!----------------------------------------------------------------------------

SUBROUTINE UCase (MyStr)
    CHARACTER (*), INTENT(InOut) :: MyStr
    INTEGER :: J, Length
    
    Length = LEN(MyStr)
    DO J = 1, Length
        IF (LGE(MyStr(J:J), "a") .AND. LLE(MyStr(J:J), "z")) THEN
            MyStr(J:J) = ACHAR (IACHAR(MyStr(J:J)) - 32)
        END IF
    END DO
END SUBROUTINE UCase

!----------------------------------------------------------------------------

Subroutine Haploid (IUnit, IOut, model, maxage, NPop, LineSkipped, &
                    LineRead, ierror, ILog)

    implicit none
    integer, intent(in) :: IUnit,IOut,LineSkipped, model, maxage, NPop, ILog
    real :: SSDtot  !, poiss1(maxage)
    integer, intent (inout) :: LineRead, ierror
    real, allocatable :: p(:),b1(:),s1(:),G1(:),L1(:), b1Scaled(:), poiss1(:)
    character reply
    real :: Adult, TotalN, totGeneration, kbarPoisson, kbarAve, growth, frac
    real :: Vk, Ne, NeLong, rNeN, rNeA
    integer :: j, m

    real :: ScaledNe


    growth = 1.0
    ALLOCATE (p(maxage))
    ALLOCATE (b1(maxage))
    ALLOCATE (s1(0:maxage))
    ALLOCATE (G1(0:maxage))
    ALLOCATE (L1(0:maxage))
    ALLOCATE (b1Scaled(maxage))
    ALLOCATE (poiss1(maxage))
    LineRead = LineSkipped
    do j=1, maxage
        LineRead = LineRead + 1
        read(IUnit, *, IOSTAT=ierror) m, s1(j), b1(j), poiss1(j)
        if (ierror <> 0) go to 999
        if (m <> j) then
            write (*, 10, advance="no") LineRead, m, s1(j), b1(j), poiss1(j)
            read (*, 15) reply
            if (reply /= 'y' .and. reply /= 'Y') stop
        end if
!            print*, m, s1(j), b1(j), poiss1(j)
    end do

    s1(0) = 1
    s1(maxage)= 0
    call LeslieOne(s1, b1, L1, G1, b1Scaled, maxage, &
                    TotalN, Adult, totGeneration, p)
!print*,'Total N=',TotalN
!print*,'Generation=',totGeneration
    frac = 1.0
    call SSDperGrp (maxage, frac, b1Scaled, G1, SSDtot, poiss1, &
                        model, growth, kbarPoisson, kbarAve, ILog, NPop)
    
    call PrintParam (IOut, model, NPop, maxage, 1, poiss1, poiss1)
    call PrintSumTable ("Haploid", IOut, maxage, NPop, s1,b1,L1, b1Scaled, p)
        Vk = SSDtot
    Ne = ScaledNe(totGeneration, Vk, model+1)
!    Ne = ScaledNe(totGeneration, denom, model+1)
    rNeN = Ne/TotalN
    rNeA = Ne/Adult

    !Now need NPop as multiplier:
    Ne = NPop*Ne
    NeLong = NPop*NeLong
    SSDtot = NPop*SSDtot
    Adult = NPop*Adult
    TotalN = NPop*TotalN
! The constants in the list of parameters here are those not used for Haploid
    call PrintSummary (IOut, maxage, NPop, 0,0, Vk, Ne, NeLong, rNeN, rNeA, &
                    TotalN, TotalN, TotalN, Adult, Adult, Adult, 0, 0, kbarAve, &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, &
                    totGeneration, model, 0, poiss1, poiss1)
999 DeALLOCATE (p)
    DeALLOCATE (b1)
    DeALLOCATE (s1)
    DeALLOCATE (G1)
    DeALLOCATE (L1)
    DeALLOCATE (b1Scaled)
    DeALLOCATE (poiss1)

10  format (/,1x, 'The age numbering is not in the order.',/, 1x, &
            'Age, survival/birth rates, Poisson factor on line', I4,':', &
            /,5x, I6, 3(4x,f8.2), /, 1x, &
            'Continue? (Y to continue, else to stop)')
15  format (A)

END SUBROUTINE Haploid

!----------------------------------------------------------------------------

Subroutine TwoSex (IUnit, IOut, model,  maxage, NPop, initialrm, &
                    LineSkipped, LineRead,  ierror, ageErr, ILog)

    implicit none
    integer, intent(in) :: IUnit, IOut, LineSkipped, model, maxage, NPop, ILog
    integer, intent (inout) :: LineRead, ierror, ageErr
    real, intent(in) :: initialrm
    real, allocatable :: s(:,:), b(:,:), Bscaled(:,:), rm(:), &
                Bmix(:,:), Pmix(:,:), Q(:), G(:, :), &
                L(:,:), pM(:), pF(:), poisson(:, :)

    real ::  totGeneration, SSD, SSDtot, Adult, AdultF, AdultM, TotalN, &
            TotalF, TotalM, rmTot, rmAdult, growth, Vk, Ne, NeLong, rNeN, &
            rNeA, kbarPoisson, kbarPoism, kbarPoisf, kbarAve, km, kf, &
            kmm, kff, kmf, kfm, Vm, Vf, Nb, Nbf, Nbm, kBarYr, kbarF, kbarM, &
            NbH, VkYr, VkF, VkM, denom
    integer :: m, j, indStop, iend
    character reply
!the following is the name of the function, needed because of implicit statement
    real :: Breeder
    real :: CrossVar, ScaledNe

    growth = 2.0
    indStop = 0
    ageErr = 0

    ALLOCATE (s(2,0:maxage))
    s = 1	!initialize
    ALLOCATE (b(2, maxage))
    ALLOCATE (Bscaled(2,maxage))
    ALLOCATE (rm(maxage))
    ALLOCATE (Bmix(0:maxage, maxage))
    ALLOCATE (Pmix(0:maxage, maxage))
    ALLOCATE (Q(0:maxage))
    ALLOCATE (G(0:maxage, maxage))
    ALLOCATE (L(2, 0:maxage))
    ALLOCATE (pM(maxage))
    ALLOCATE (pF(maxage))
    ALLOCATE (poisson(2,maxage))
    poisson = 1.0	!initialize
    do j=1, maxage
        LineRead = LineRead + 1
        iend = 0
        if (model > 0) then
            read(IUnit, *, IOSTAT=ierror) m, s(1,j), b(1,j), poisson(1,j), &
                            s(2,j), b(2,j), poisson(2,j), rm(j)
        else
            read(IUnit, *, IOSTAT=ierror) m, s(1,j), b(1,j), poisson(1,j), &
                            s(2,j), b(2,j), poisson(2,j)
            rm = initialrm	!initialize array rm
        end if
        if (ierror <> 0) go to 999
        if (m <> j) then
            write (*, 10, advance="no") LineRead, m, &
                                s(1,j), b(1, j), poisson(1,j)
            read (*, 15) reply
            if (reply /= 'y' .and. reply /= 'Y') stop
        end if
    end do

! This is for no sex change, kmm = expected male progeny/male parent,
! kmf = expected female progeny/male parent. Similar for kff, kfm.
    kmm = 1.0
    kff = 1.0
    kmf = (1.0-initialrm)/initialrm
    kfm = 1.0/kmf

! to ensure consistency, set both survival rates for individuals at
! maxage be 0 (just in case input was wrong for having nonzero):
    s(1,maxage) = 0
    s(2,maxage) = 0

    call LeslieCalcul(pM, pF, s, b, Bmix, rm, maxage, L, totGeneration, &   !9
                    indStop, Q, G, poisson, Pmix, TotalN, TotalF, TotalM, & !8
                    Adult, AdultF, AdultM, rmTot, rmAdult, Bscaled, model, &    !7
                    growth, kBarYr, kbarF, kbarM, VkYr, VkF, VkM, ILog) !8
!if (ILog > 0) write (ILog, '(/A,I4/)') "IndStop = ", indStop
    Nbf = Breeder(NPop, TotalF, 1.0, kbarF, VkF, -1.0)
    Nbm = Breeder(NPop, TotalM, 1.0, kbarM, VkM, -1.0)
    Nb = Breeder(NPop, TotalN, 1.0, kBarYr, VkYr, -2.0)
    NbH = (4 * Nbf * Nbm)/(Nbf + Nbm)
!Nb = (kBarYr*TotalN*NPop - 2)/(kBarYr - 1 + (VkYr/kBarYr))
    if (indStop > 0 .and. indStop < maxage) then
        ierror = 1
        ageErr = indStop+1
        return
    end if
!print*,'kBarYr= ', kBarYr
!print*,'VkYr = ', VkYr
!print*,'Nbf =', Nbf, ', Nbm = ', Nbm
!print*,'Nb per year =', Nb, ', from Nbf, Nbm = ', NbH
    SSDtot = 0
    ! Do for favorite sex in majority first:
    If (indStop < maxage) then

        if (ILog == 0) go to 101
        if (model==1) then
            write (ILog,'(/,">>", I2,a)') maxage,'F'
        else
            write (ILog,'(/,">>", I2,a)') maxage,'M'
        end if
101 continue

        call SSDperGrp (maxage, Q(maxage), Bmix(maxage,:), G(maxage,:), SSD, &
            Pmix(maxage,:), model, growth, kbarPoisson, kbarAve, ILog, NPop)
        SSDtot = SSDtot + SSD
        if (model == 0) then    !no sex change: this is for female
            kbarPoisf = kbarPoisson
            kf = kbarAve
            Vf = SSD
!print*,'total female=',NPop*TotalN*(1-rmTot)
!print*,'Nbf=',Nbf
        end if

!Test: This should be column for kbar
!print*,'First group:'
!temp=0
!do i=1,maxage
!temp=temp+BMix(maxage,i)
!print*,temp
!end do            
            
    End If
    Do j = indStop, 0, -1
    !the real SSD's should be multiplied with NPop if want as outputs.
        if (ILog == 0) go to 202
        if (Q(j)>0) then
            if (model==1) then
                write (ILog,'(/,">>", 2(I2,a))') j,'F', (maxage-j),'M'
            else
                write (ILog,'(/,">>", 2(I2,a))') j,'M', (maxage-j),'F'
            end if
        end if
202 continue
!
        call SSDperGrp (maxage, Q(j), Bmix(j,:), G(j,:), SSD, Pmix(j,:), &
                        model, growth, kbarPoisson, kbarAve, ILog, NPop)
        SSDtot = SSDtot + SSD
        if (model == 0) then    !no sex change: this is for male
                                    !indStop should be 0
            kbarPoism = kbarPoisson
            km = kbarAve
            Vm = SSD
        end if

!print*,'Next group:'
!temp=0
!do i=1,maxage
!temp=temp+BMix(j,i)
!print*,temp
!end do            

    End Do



            
! check:
!        totmale = 0    
!        do j = 0, maxage
!            do t = j+1, maxage
!                totmale = totmale + G(j, t)*Q(j)
!            end do
!        end do
!        totmale = totmale * NPop
!        print*, 'total male = ', totmale
!        write (IOut, '(/,a)') "Group fractions: "
!        do j=0, maxage
!            write (IOut, 110) Q(j)
!        end do
!        write (IOut, '(/,a, f8.5)') "Generation = ", totGeneration
!        write (IOut, '(/,a)') "Accumulated survival rates: "
!        do j=0, maxage
!            write (IOut, 120) L(1,j), L(2,j)
!        end do
!110         format (10x,f10.6)
!120 format (2(10x,f10.6))

    call PrintParam (IOut, model, NPop, maxage, initialrm, &
                    poisson(1,:), poisson(2,:))
    call PrintSumTable ("Females", IOut, maxage, NPop, s(1,:), b(1,:), &
                        L(1,:), bScaled(1,:), pF)
    call PrintSumTable ("Males", IOut, maxage, NPop, s(2,:), b(2,:), &
                        L(2,:), bScaled(2,:), pM)

    If (model == 0) then
        kbarAve = initialrm*km + (1.0-initialrm)*kf
        Vk = initialrm*Vm + (1.0-initialrm)*Vf + &
                    initialrm*(1.0-initialrm)*(km - kf)*(km - kf)
        denom = CrossVar (Vm, Vf, kbarPoism, kbarPoisf, &
                    km, kf, kmm, kmf, kff, kfm, initialrm)
        NeLong = ScaledNe(totGeneration, denom, model)
    else
        Vk = SSDtot
    end if
    Ne = ScaledNe(totGeneration, Vk, model+1)
!    Ne = ScaledNe(totGeneration, denom, model+1)
    rNeN = Ne/TotalN
    rNeA = Ne/Adult

    !Now need NPop as multiplier:
    Ne = NPop*Ne
    NeLong = NPop*NeLong
    SSDtot = NPop*SSDtot
    Adult = NPop*Adult
    AdultF = NPop*AdultF
    AdultM = NPop*AdultM
    TotalN = NPop*TotalN
    TotalF = NPop*TotalF
    TotalM = NPop*TotalM
    call PrintSummary (IOut, maxage, NPop, Vm, Vf, Vk, Ne, NeLong, rNeN, &
                        rNeA, TotalN, TotalF, TotalM,  Adult, AdultF, AdultM, &
                        km, kf, kbarAve, &
                        Nb, Nbf, Nbm, kbarM, kbarF, kBarYr, VkM, VkF, VkYr, &
                        totGeneration, model, indStop, &
                        poisson(1,:), poisson(2,:))
999 DeALLOCATE (s)
    DeALLOCATE (b)
    DeALLOCATE (Bscaled)
    DeALLOCATE (rm)
    DeALLOCATE (Bmix)
    DeALLOCATE (Pmix)
    DeALLOCATE (Q)
    DeALLOCATE (G)
    DeALLOCATE (L)
    DeALLOCATE (pM)
    DeALLOCATE (pF)
    DeALLOCATE (poisson)

10  format (/,1x, 'The age numbering is not in the order.',/, 1x, &
            'Age, survival/birth rates, Poisson factor on line', I4,':', &
            /,5x, I6, 3(4x,f8.2), /, 1x, 'Continue (and ignore from now on)? ')
15  format (A)
!20  format (1x,"Predetermined fraction male at age ", I3, &
!            " does not allow sex change.", /,1x,"Sex change stops at age", I3)

End Subroutine TwoSex